;------------------------------------------------------------------------------
;-	  ATMEL Microcontroller Software Support  -   ROUSSET -
;------------------------------------------------------------------------------
;- File source				: turbo_dma.s
;- Translator				: ARM Software Development Toolkit V2.11
;-
;- Treatment				: Turbo DMA
;-                            Copy 4 word at each FIQ
;- Imported Resources		: none
;- Exported Resources		: DmaTurboSet   DmaTurboGet   DmaTurboFiq
;-
;- 1.0 SDC 30/07/98			: Creation
;------------------------------------------------------------------------------

			AREA			AT91Lib, CODE, READONLY

;------------------------- List of Included Files -----------------------------
			INCLUDE			arm.inc

;------------------------- List of constants and types ------------------------
;- None

;------------------------- List of Imported resources -------------------------
;- None

;------------------------- List of Internal resources -------------------------
;- None

;------------------------- List of External resources -------------------------
;------------------------------------------------------------------------------
;- Function				: DmaTurboSet
;- Treatments			: Initialize registers for turbo DMA and enable DMA
;-                        (This function must be called in PRIVILEDGE mode !)
;- Input Parameters		: r0 = 'source' : source buffer pointer
;-                        r1 = 'destination' : destination buffer pointer
;- Output Parameters	: none
;- Registers lost		: r2, r3
;- Called Functions		: none
;- Called Macros		: none
;------------------------------------------------------------------------------
			EXPORT		DmaTurboSet
;- Begin
DmaTurboSet
;- | Switch to FIQ mode and save entry mode 
			mrs		r2, CPSR
			mov		r3, #ARM_MODE_FIQ:OR:I_BIT:OR:F_BIT
			msr		CPSR, r3

;- | Memorize source buffer pointer (r12_fiq = 'src_pt')
			mov		r12, r0

;- | Memorize destination buffer pointer (r13_fiq = 'dest_pt')
			mov		r13, r1

;- | Enable DMA (Clear bit F)
			bic		r2, r2, #F_BIT

;- | Restore entry mode
			msr		CPSR, r2

;- | End
			mov		pc,lr

;------------------------------------------------------------------------------
;- Function				: DmaTurboGet
;- Treatments			: Disable DMA when the word count is reached
;-                        (This function must be called in PRIVILEDGE mode !)
;- Input Parameters		: r0 = 'source' : source buffer pointer
;-                        r1 = 'size' : word number to transfer
;- Output Parameters	: r0 = transferred word count
;- Registers lost		: r2, r3
;- Called Functions		: none
;- Called Macros		: none
;------------------------------------------------------------------------------
			EXPORT		DmaTurboGet
;- Begin
DmaTurboGet
;- | Switch to FIQ mode and save entry mode 
			mrs		r2, CPSR
			mov		r3, #ARM_MODE_FIQ:OR:I_BIT:OR:F_BIT
			msr		CPSR, r3

;- | Calculate number of transferred words 
			sub		r0, r12, r0
			mov		r0, r0, lsr #2

;- | Disable DMA (Clear bit F) if number of transferred words >= size
			cmp			r0, r1
			bicle		r2, r2, #F_BIT

;- | Restore entry mode
			msr		CPSR, r2

;- End
			mov		pc,lr

;------------------------------------------------------------------------------
;- Function				: DmaTurboFiq
;- Treatments			: FIQ transfert interrupt
;-                        Copy 4 words from 'src_pt' to 'dest_pt' 
;- 						  and increment pointers accordingly.
;- Input Parameters		: none
;- Output Parameters	: none
;- Registers lost		: none
;- Called Functions		: none
;- Called Macros		: none
;------------------------------------------------------------------------------
			EXPORT		DmaTurboFiq
;- Begin
DmaTurboFiq

;- | *dest_pt <- *src_pt (multiple load and store 4 words with post increment)
			ldmia	r12!, {r8-r11}
			stmia	r13!, {r8-r11}

;- End
			subs		pc, lr, #4

			END
